@@ -14,10 +14,11 @@ Global minimization using the ``brute`` method (a.k.a. grid search)
 # (see below).
 import copy
 
+from matplotlib.colors import LogNorm
 import matplotlib.pyplot as plt
 import numpy as np
 
-from lmfit import Minimizer, Parameters, fit_report
+from lmfit import Minimizer, create_params, fit_report
 
 ###############################################################################
 # Let's start with the example given in the documentation of SciPy:
@@ -25,29 +26,32 @@ from lmfit import Minimizer, Parameters, fit_report
 # "We illustrate the use of brute to seek the global minimum of a function of
 # two variables that is given as the sum of a positive-definite quadratic and
 # two deep “Gaussian-shaped” craters. Specifically, define the objective
-# function f as the sum of three other functions, ``f = f1 + f2 + f3``. We
+# function ``f`` as the sum of three other functions, ``f = f1 + f2 + f3``. We
 # suppose each of these has a signature ``(z, *params), where z = (x, y)``,
 # and params and the functions are as defined below."
 #
 # First, we create a set of Parameters where all variables except ``x`` and
 # ``y`` are given fixed values.
-params = Parameters()
-params.add_many(
-        ('a', 2, False),
-        ('b', 3, False),
-        ('c', 7, False),
-        ('d', 8, False),
-        ('e', 9, False),
-        ('f', 10, False),
-        ('g', 44, False),
-        ('h', -1, False),
-        ('i', 2, False),
-        ('j', 26, False),
-        ('k', 1, False),
-        ('l', -2, False),
-        ('scale', 0.5, False),
-        ('x', 0.0, True),
-        ('y', 0.0, True))
+# Just as in the documentation we will do a grid search between ``-4`` and
+# ``4`` and use a stepsize of ``0.25``. The bounds can be set as usual with
+# the ``min`` and ``max`` attributes, and the stepsize is set using
+# ``brute_step``.
+
+params = create_params(a=dict(value=2, vary=False),
+                       b=dict(value=3, vary=False),
+                       c=dict(value=7, vary=False),
+                       d=dict(value=8, vary=False),
+                       e=dict(value=9, vary=False),
+                       f=dict(value=10, vary=False),
+                       g=dict(value=44, vary=False),
+                       h=dict(value=-1, vary=False),
+                       i=dict(value=2, vary=False),
+                       j=dict(value=26, vary=False),
+                       k=dict(value=1, vary=False),
+                       l=dict(value=-2, vary=False),
+                       scale=dict(value=0.5, vary=False),
+                       x=dict(value=0.0, vary=True, min=-4, max=4, brute_step=0.25),
+                       y=dict(value=0.0, vary=True, min=-4, max=4, brute_step=0.25))
 
 ###############################################################################
 # Second, create the three functions and the objective function:
@@ -56,33 +60,26 @@ params.add_many(
 def f1(p):
     par = p.valuesdict()
     return (par['a'] * par['x']**2 + par['b'] * par['x'] * par['y'] +
-            par['c'] * par['y']**2 + par['d']*par['x'] + par['e']*par['y'] + par['f'])
+            par['c'] * par['y']**2 + par['d']*par['x'] + par['e']*par['y'] +
+            par['f'])
 
 
 def f2(p):
     par = p.valuesdict()
     return (-1.0*par['g']*np.exp(-((par['x']-par['h'])**2 +
-            (par['y']-par['i'])**2) / par['scale']))
+                                   (par['y']-par['i'])**2) / par['scale']))
 
 
 def f3(p):
     par = p.valuesdict()
     return (-1.0*par['j']*np.exp(-((par['x']-par['k'])**2 +
-            (par['y']-par['l'])**2) / par['scale']))
+                                   (par['y']-par['l'])**2) / par['scale']))
 
 
 def f(params):
     return f1(params) + f2(params) + f3(params)
 
 
-###############################################################################
-# Just as in the documentation we will do a grid search between ``-4`` and
-# ``4`` and use a stepsize of ``0.25``. The bounds can be set as usual with
-# the ``min`` and ``max`` attributes, and the stepsize is set using
-# ``brute_step``.
-params['x'].set(min=-4, max=4, brute_step=0.25)
-params['y'].set(min=-4, max=4, brute_step=0.25)
-
 ###############################################################################
 # Performing the actual grid search is done with:
 fitter = Minimizer(f, params)
@@ -91,7 +88,7 @@ result = fitter.minimize(method='brute')
 ###############################################################################
 # , which will increment ``x`` and ``y`` between ``-4`` in increments of
 # ``0.25`` until ``4`` (not inclusive).
-grid_x, grid_y = [np.unique(par.ravel()) for par in result.brute_grid]
+grid_x, grid_y = (np.unique(par.ravel()) for par in result.brute_grid)
 print(grid_x)
 
 ###############################################################################
@@ -104,35 +101,36 @@ print(grid_x)
 print(result.brute_x0)
 
 ###############################################################################
-# ``result.brute_fval`` -- Function value at the point x0.
+# ``result.brute_fval`` -- Function value at the point ``x0``.
 print(result.brute_fval)
 
 ###############################################################################
 # ``result.brute_grid`` -- Representation of the evaluation grid. It has the
-# same length as x0.
+# same length as ``x0``.
 print(result.brute_grid)
 
 ###############################################################################
 # ``result.brute_Jout`` -- Function values at each point of the evaluation
-# grid, i.e., Jout = func(\*grid).
+# grid, i.e., ``Jout = func(*grid)``.
 print(result.brute_Jout)
 
 ###############################################################################
-# **Reassuringly, the obtained results are indentical to using the method in
+# **Reassuringly, the obtained results are identical to using the method in
 # SciPy directly!**
 
 ###############################################################################
 # Example 2: fit of a decaying sine wave
 #
-# In this example, will explain some of the options ot the algorithm.
+# In this example, we will explain some of the options of the algorithm.
 #
-# We start off by generating some synthetic data with noise for a decaying
-# sine wave, define an objective function and create a Parameter set.
+# We start off by generating some synthetic data with noise for a decaying sine
+# wave, define an objective function, and create/initialize a Parameter set.
 x = np.linspace(0, 15, 301)
 np.random.seed(7)
 noise = np.random.normal(size=x.size, scale=0.2)
 data = (5. * np.sin(2*x - 0.1) * np.exp(-x*x*0.025) + noise)
-plt.plot(x, data, 'b')
+plt.plot(x, data, 'o')
+plt.show()
 
 
 def fcn2min(params, x, data):
@@ -145,21 +143,15 @@ def fcn2min(params, x, data):
     return model - data
 
 
-# create a set of Parameters
-params = Parameters()
-params.add('amp', value=7, min=2.5)
-params.add('decay', value=0.05)
-params.add('shift', value=0.0, min=-np.pi/2., max=np.pi/2)
-params.add('omega', value=3, max=5)
-
 ###############################################################################
 # In contrast to the implementation in SciPy (as shown in the first example),
-# varying parameters do not need to have finite bounds in lmfit. However, in
-# that case they **do** need the ``brute_step`` attribute specified, so let's
-# do that:
-params['amp'].set(brute_step=0.25)
-params['decay'].set(brute_step=0.005)
-params['omega'].set(brute_step=0.25)
+# varying parameters do not need to have finite bounds in lmfit. However, if a
+# parameter does not have finite bounds, then it does need a ``brute_step``
+# attribute specified:
+params = create_params(amp=dict(value=7, min=2.5, brute_step=0.25),
+                       decay=dict(value=0.05, brute_step=0.005),
+                       shift=dict(value=0.0, min=-np.pi/2., max=np.pi/2),
+                       omega=dict(value=3, max=5, brute_step=0.25))
 
 ###############################################################################
 # Our initial parameter set is now defined as shown below and this will
@@ -167,10 +159,12 @@ params['omega'].set(brute_step=0.25)
 params.pretty_print()
 
 ###############################################################################
-# First, we initialize a Minimizer and perform the grid search:
+# First, we initialize a ``Minimizer`` and perform the grid search:
 fitter = Minimizer(fcn2min, params, fcn_args=(x, data))
 result_brute = fitter.minimize(method='brute', Ns=25, keep=25)
 
+print(fit_report(result_brute))
+
 ###############################################################################
 # We used two new parameters here: ``Ns`` and ``keep``. The parameter ``Ns``
 # determines the \'number of grid points along the axes\' similarly to its usage
@@ -184,43 +178,45 @@ result_brute = fitter.minimize(method='brute', Ns=25, keep=25)
 par_name = 'shift'
 indx_shift = result_brute.var_names.index(par_name)
 grid_shift = np.unique(result_brute.brute_grid[indx_shift].ravel())
-print("parameter = {}\nnumber of steps = {}\ngrid = {}".format(par_name,
-      len(grid_shift), grid_shift))
+print(f"parameter = {par_name}\nnumber of steps = {len(grid_shift)}\ngrid = {grid_shift}")
 
 ###############################################################################
 # If finite bounds are not set for a certain parameter then the user **must**
 # specify ``brute_step`` - three more scenarios are considered here:
 #
-# **(2)** lower bound (min) and brute_step are specified:
-# range = (min, min + Ns * brute_step, brute_step)
+# **(2)** lower bound ``(min``) and ``brute_step`` are specified:
+# ``range = (min, min + Ns * brute_step, brute_step)``
 par_name = 'amp'
 indx_shift = result_brute.var_names.index(par_name)
 grid_shift = np.unique(result_brute.brute_grid[indx_shift].ravel())
-print("parameter = {}\nnumber of steps = {}\ngrid = {}".format(par_name, len(grid_shift), grid_shift))
+print(f"parameter = {par_name}\nnumber of steps = {len(grid_shift)}\ngrid = {grid_shift}")
 
 ###############################################################################
-# **(3)** upper bound (max) and brute_step are specified:
-# range = (max - Ns * brute_step, max, brute_step)
+# **(3)** upper bound (``max``) and ``brute_step`` are specified:
+# ``range = (max - Ns * brute_step, max, brute_step)``
 par_name = 'omega'
 indx_shift = result_brute.var_names.index(par_name)
 grid_shift = np.unique(result_brute.brute_grid[indx_shift].ravel())
-print("parameter = {}\nnumber of steps = {}\ngrid = {}".format(par_name, len(grid_shift), grid_shift))
+print(f"parameter = {par_name}\nnumber of steps = {len(grid_shift)}\ngrid = {grid_shift}")
 
 ###############################################################################
-# **(4)** numerical value (value) and brute_step are specified:
-# range = (value - (Ns//2) * brute_step, value + (Ns//2) * brute_step, brute_step)
+# **(4)** numerical value (``value``) and ``brute_step`` are specified:
+# ``range = (value - (Ns//2) * brute_step, value + (Ns//2) * brute_step, brute_step)``
 par_name = 'decay'
 indx_shift = result_brute.var_names.index(par_name)
 grid_shift = np.unique(result_brute.brute_grid[indx_shift].ravel())
-print("parameter = {}\nnumber of steps = {}\ngrid = {}".format(par_name, len(grid_shift), grid_shift))
+print(f"parameter = {par_name}\nnumber of steps = {len(grid_shift)}\ngrid = {grid_shift}")
 
 ###############################################################################
 # The ``MinimizerResult`` contains all the usual best-fit parameters and
 # fitting statistics. For example, the optimal solution from the grid search
 # is given below together with a plot:
 print(fit_report(result_brute))
-plt.plot(x, data, 'b')
-plt.plot(x, data + fcn2min(result_brute.params, x, data), 'r--')
+
+###############################################################################
+plt.plot(x, data, 'o')
+plt.plot(x, data + fcn2min(result_brute.params, x, data), '--')
+plt.show()
 
 ###############################################################################
 # We can see that this fit is already very good, which is what we should expect
@@ -230,12 +226,12 @@ plt.plot(x, data + fcn2min(result_brute.params, x, data), 'r--')
 # In a more realistic, complicated example the ``brute`` method will be used
 # to get reasonable values for the parameters and perform another minimization
 # (e.g., using ``leastsq``) using those as starting values. That is where the
-# `keep`` parameter comes into play: it determines the "number of best
+# ``keep`` parameter comes into play: it determines the "number of best
 # candidates from the brute force method that are stored in the ``candidates``
 # attribute". In the example above we store the best-ranking 25 solutions (the
 # default value is ``50`` and storing all the grid points can be accomplished
 # by choosing ``all``). The ``candidates`` attribute contains the parameters
-# and ``chisqr`` from the brute force method as a namedtuple,
+# and ``chisqr`` from the brute force method as a ``namedtuple``,
 # ``(‘Candidate’, [‘params’, ‘score’])``, sorted on the (lowest) ``chisqr``
 # value. To access the values for a particular candidate one can use
 # ``result.candidate[#].params`` or ``result.candidate[#].score``, where a
@@ -275,10 +271,10 @@ print(fit_report(best_result))
 # As expected the parameters have not changed significantly as they were
 # already very close to the "real" values, which can also be appreciated from
 # the plots below.
-plt.plot(x, data, 'b')
-plt.plot(x, data + fcn2min(result_brute.params, x, data), 'r--',
+plt.plot(x, data, 'o')
+plt.plot(x, data + fcn2min(result_brute.params, x, data), '-',
          label='brute')
-plt.plot(x, data + fcn2min(best_result.params, x, data), 'g--',
+plt.plot(x, data + fcn2min(best_result.params, x, data), '--',
          label='brute followed by leastsq')
 plt.legend()
 
@@ -311,10 +307,8 @@ def plot_results_brute(result, best_vals=True, varlabels=None,
     output : str, optional
         Name of the output PDF file (default is 'None')
     """
-    from matplotlib.colors import LogNorm
-
     npars = len(result.var_names)
-    fig, axes = plt.subplots(npars, npars)
+    _fig, axes = plt.subplots(npars, npars)
 
     if not varlabels:
         varlabels = result.var_names
@@ -337,7 +331,7 @@ def plot_results_brute(result, best_vals=True, varlabels=None,
                 if i == 0:
                     axes[0, 0].axis('off')
                 ax = axes[i, j+1]
-                red_axis = tuple([a for a in range(npars) if a != i])
+                red_axis = tuple(a for a in range(npars) if a != i)
                 ax.plot(np.unique(result.brute_grid[i]),
                         np.minimum.reduce(result.brute_Jout, axis=red_axis),
                         'o', ms=3)
@@ -351,14 +345,14 @@ def plot_results_brute(result, best_vals=True, varlabels=None,
             # parameter vs chi2 profile on the left
             elif j == 0 and i > 0:
                 ax = axes[i, j]
-                red_axis = tuple([a for a in range(npars) if a != i])
+                red_axis = tuple(a for a in range(npars) if a != i)
                 ax.plot(np.minimum.reduce(result.brute_Jout, axis=red_axis),
                         np.unique(result.brute_grid[i]), 'o', ms=3)
                 ax.invert_xaxis()
                 ax.set_ylabel(varlabels[i])
                 if i != npars-1:
                     ax.set_xticks([])
-                elif i == npars-1:
+                else:
                     ax.set_xlabel(r'$\chi^{2}$')
                 if best_vals:
                     ax.axhline(best_vals[par1].value, ls='dashed', color='r')
@@ -366,7 +360,7 @@ def plot_results_brute(result, best_vals=True, varlabels=None,
             # contour plots for all combinations of two parameters
             elif j > i:
                 ax = axes[j, i+1]
-                red_axis = tuple([a for a in range(npars) if a != i and a != j])
+                red_axis = tuple(a for a in range(npars) if a not in (i, j))
                 X, Y = np.meshgrid(np.unique(result.brute_grid[i]),
                                    np.unique(result.brute_grid[j]))
                 lvls1 = np.linspace(result.brute_Jout.min(),
@@ -383,7 +377,7 @@ def plot_results_brute(result, best_vals=True, varlabels=None,
                     ax.plot(best_vals[par1].value, best_vals[par2].value, 'rs', ms=3)
                 if j != npars-1:
                     ax.set_xticks([])
-                elif j == npars-1:
+                else:
                     ax.set_xlabel(varlabels[i])
                 if j - i >= 2:
                     axes[i, j].axis('off')
@@ -395,3 +389,4 @@ def plot_results_brute(result, best_vals=True, varlabels=None,
 ###############################################################################
 # and finally, to generated the figure:
 plot_results_brute(result_brute, best_vals=True, varlabels=None)
+plt.show()
