@@ -17,10 +17,11 @@ x = np.linspace(-10, 10, 201)
 amp, cen, wid = 3.4, 1.8, 0.5
 
 y = amp * np.exp(-(x-cen)**2 / (2*wid**2)) / (np.sqrt(2*np.pi)*wid)
+np.random.seed(2021)
 y = y + np.random.normal(size=x.size, scale=0.01)
 
 ###############################################################################
-# Define the ExpressionModel and perform the fit:
+# Define the ``ExpressionModel`` and perform the fit:
 gmod = ExpressionModel("amp * exp(-(x-cen)**2 /(2*wid**2))/(sqrt(2*pi)*wid)")
 result = gmod.fit(y, x=x, amp=5, cen=5, wid=1)
 
@@ -28,8 +29,8 @@ result = gmod.fit(y, x=x, amp=5, cen=5, wid=1)
 # this results in the following output:
 print(result.fit_report())
 
-plt.plot(x, y, 'bo')
-plt.plot(x, result.init_fit, 'k--', label='initial fit')
-plt.plot(x, result.best_fit, 'r-', label='best fit')
-plt.legend(loc='best')
-plt.show()
+###############################################################################
+plt.plot(x, y, 'o')
+plt.plot(x, result.init_fit, '--', label='initial fit')
+plt.plot(x, result.best_fit, '-', label='best fit')
+plt.legend()
