@@ -1,26 +1,34 @@
+import sys
 import unittest
 
 import numpy as np
 from numpy import pi
-from numpy.testing import (assert_, assert_allclose, assert_almost_equal,
-                           assert_equal, dec)
+from numpy.testing import assert_allclose, assert_almost_equal, assert_equal
 import pytest
+from scipy.optimize import rosen_der
 from uncertainties import ufloat
 
 from lmfit import Minimizer, Parameters, minimize
 from lmfit.lineshapes import gaussian
 from lmfit.minimizer import (HAS_EMCEE, SCALAR_METHODS, MinimizerResult,
-                             _nan_policy)
+                             coerce_float64)
+from lmfit.models import SineModel
+
+try:
+    import numdifftools  # noqa: F401
+    HAS_NUMDIFFTOOLS = True
+except ImportError:
+    HAS_NUMDIFFTOOLS = False
 
 
 def check(para, real_val, sig=3):
     err = abs(para.value - real_val)
-    assert(err < sig * para.stderr)
+    assert err < sig * para.stderr
 
 
 def check_wo_stderr(para, real_val, sig=0.1):
     err = abs(para.value - real_val)
-    assert(err < sig)
+    assert err < sig
 
 
 def check_paras(para_fit, para_real, sig=3):
@@ -79,7 +87,7 @@ def test_lbfgsb():
         model = amp * np.sin(shift + x / per) * np.exp(-x * x * decay * decay)
         if data is None:
             return model
-        return (model - data)
+        return model - data
 
     n = 2500
     xmin = 0.
@@ -150,7 +158,7 @@ def test_peakfit():
         model = g1 + g2
         if data is None:
             return model
-        return (model - data)
+        return model - data
 
     n = 601
     xmin = 0.
@@ -219,18 +227,15 @@ def test_scalar_minimize_has_no_uncertainties():
 
     mini = Minimizer(fcn2min, params, fcn_args=(x, data))
     out = mini.minimize()
-    assert_(np.isfinite(out.params['amp'].stderr))
+    assert np.isfinite(out.params['amp'].stderr)
     assert out.errorbars
     out2 = mini.minimize(method='nelder-mead')
-    assert_(out2.params['amp'].stderr is None)
-    assert_(out2.params['decay'].stderr is None)
-    assert_(out2.params['shift'].stderr is None)
-    assert_(out2.params['omega'].stderr is None)
-    assert_(out2.params['amp'].correl is None)
-    assert_(out2.params['decay'].correl is None)
-    assert_(out2.params['shift'].correl is None)
-    assert_(out2.params['omega'].correl is None)
-    assert not out2.errorbars
+
+    for par in ('amp', 'decay', 'shift', 'omega'):
+        assert HAS_NUMDIFFTOOLS == (out2.params[par].stderr is not None)
+        assert HAS_NUMDIFFTOOLS == (out2.params[par].correl is not None)
+
+    assert HAS_NUMDIFFTOOLS == out2.errorbars
 
 
 def test_scalar_minimize_reduce_fcn():
@@ -282,7 +287,7 @@ def test_multidimensional_fit_GH205():
                                           np.cos(yv * lambda2))
 
     data = f(xv, yv, 0.3, 3)
-    assert_(data.ndim, 2)
+    assert data.ndim, 2
 
     def fcn2min(params, xv, yv, data):
         """model decaying sine wave, subtract data"""
@@ -313,6 +318,32 @@ def test_ufloat():
     assert_allclose(y.std_dev, 0.0, rtol=1.e-7)
 
 
+def test_stderr_propagation():
+    """Test propagation of uncertainties to constraint expressions."""
+    model = SineModel()
+    params = model.make_params(amplitude=1, frequency=9.0, shift=0)
+    params.add("period", expr="1/frequency")
+    params.add("period_2a", expr="2*period")
+    params.add("period_2b", expr="2/frequency")
+    params.add("thing_1", expr="shift + frequency")
+    params.add("thing_2", expr="shift + 1/period")
+
+    np.random.seed(3)
+    xs = np.linspace(0, 1, 51)
+    ys = np.sin(7.45 * xs) + 0.01 * np.random.normal(size=xs.shape)
+
+    result = model.fit(ys, x=xs, params=params)
+
+    opars = result.params
+    assert_allclose(opars['period'].stderr, 1.1587e-04, rtol=1.e-3)
+    assert_allclose(opars['period_2b'].stderr, 2.3175e-04, rtol=1.e-3)
+    assert_allclose(opars['thing_1'].stderr, 0.0037291, rtol=1.e-3)
+
+    assert_allclose(opars['period_2a'].stderr, 2*opars['period'].stderr, rtol=1.e-5)
+    assert_allclose(opars['period_2b'].stderr, opars['period_2a'].stderr, rtol=1.e-5)
+    assert_allclose(opars['thing_1'].stderr, opars['thing_2'].stderr, rtol=1.e-5)
+
+
 class CommonMinimizerTest(unittest.TestCase):
 
     def setUp(self):
@@ -365,7 +396,7 @@ class CommonMinimizerTest(unittest.TestCase):
 
         # but only if a parameter is not fixed
         self.fit_params['decay'].vary = False
-        self.mini.scalar_minimize(method='differential_evolution', maxiter=1)
+        self.mini.scalar_minimize(method='differential_evolution', max_nfev=1)
 
     def test_scalar_minimizers(self):
         # test all the scalar minimizers
@@ -388,13 +419,15 @@ class CommonMinimizerTest(unittest.TestCase):
         for para, true_para in zip(out.params.values(), self.p_true.values()):
             check_wo_stderr(para, true_para.value, sig=sig)
 
-    def test_nan_policy(self):
+    def test_coerce_float64(self):
         # check that an error is raised if there are nan in
         # the data returned by userfcn
         self.data[0] = np.nan
 
         for method in SCALAR_METHODS:
-            if method == 'differential_evolution':
+            if method == 'cobyla' and sys.platform == 'darwin':
+                pytest.xfail("this aborts Python on macOS...")
+            elif method == 'differential_evolution':
                 pytest.raises(RuntimeError, self.mini.scalar_minimize,
                               SCALAR_METHODS[method])
             else:
@@ -411,34 +444,32 @@ class CommonMinimizerTest(unittest.TestCase):
         for para, true_para in zip(res.params.values(), self.p_true.values()):
             check_wo_stderr(para, true_para.value, sig=0.15)
 
-    def test_nan_policy_function(self):
+    def test_coerce_float64_function(self):
         a = np.array([0, 1, 2, 3, np.nan])
-        pytest.raises(ValueError, _nan_policy, a)
-        assert_(np.isnan(_nan_policy(a, nan_policy='propagate')[-1]))
-        assert_equal(_nan_policy(a, nan_policy='omit'), [0, 1, 2, 3])
+        pytest.raises(ValueError, coerce_float64, a)
+        assert np.isnan(coerce_float64(a, nan_policy='propagate')[-1])
+        assert_equal(coerce_float64(a, nan_policy='omit'), [0, 1, 2, 3])
 
         a[-1] = np.inf
-        pytest.raises(ValueError, _nan_policy, a)
-        assert_(np.isposinf(_nan_policy(a, nan_policy='propagate')[-1]))
-        assert_equal(_nan_policy(a, nan_policy='omit'), [0, 1, 2, 3])
-        assert_equal(_nan_policy(a, handle_inf=False), a)
+        pytest.raises(ValueError, coerce_float64, a)
+        assert np.isposinf(coerce_float64(a, nan_policy='propagate')[-1])
+        assert_equal(coerce_float64(a, nan_policy='omit'), [0, 1, 2, 3])
+        assert_equal(coerce_float64(a, handle_inf=False), a)
 
-    @dec.slow
     def test_emcee(self):
         # test emcee
         if not HAS_EMCEE:
-            return True
+            return
 
         np.random.seed(123456)
         out = self.mini.emcee(nwalkers=100, steps=200, burn=50, thin=10)
 
         check_paras(out.params, self.p_true, sig=3)
 
-    @dec.slow
     def test_emcee_method_kwarg(self):
         # test with emcee as method keyword argument
         if not HAS_EMCEE:
-            return True
+            return
         np.random.seed(123456)
         out = self.mini.minimize(method='emcee',
                                  nwalkers=50, steps=200,
@@ -454,13 +485,12 @@ class CommonMinimizerTest(unittest.TestCase):
                                             is_weighted=False)
         assert out_unweighted.method == 'emcee'
 
-    @dec.slow
     def test_emcee_multiprocessing(self):
         # test multiprocessing runs
         raise pytest.skip("Pytest fails with multiprocessing")
         pytest.importorskip("dill")
         if not HAS_EMCEE:
-            return True
+            return
         self.mini.emcee(steps=50, workers=4, nwalkers=20)
 
     def test_emcee_bounds_length(self):
@@ -469,18 +499,17 @@ class CommonMinimizerTest(unittest.TestCase):
         # are the right lengths for comparison. This can be done
         # if nvarys != nparams
         if not HAS_EMCEE:
-            return True
+            return
         self.mini.params['amp'].vary = False
         self.mini.params['period'].vary = False
         self.mini.params['shift'].vary = False
 
         self.mini.emcee(steps=10)
 
-    @dec.slow
     def test_emcee_partial_bounds(self):
         # mcmc with partial bounds
         if not HAS_EMCEE:
-            return True
+            return
 
         np.random.seed(123456)
         # test mcmc output vs lm, some parameters not bounded
@@ -493,7 +522,7 @@ class CommonMinimizerTest(unittest.TestCase):
     def test_emcee_init_with_chain(self):
         # can you initialise with a previous chain
         if not HAS_EMCEE:
-            return True
+            return
 
         out = self.mini.emcee(nwalkers=100, steps=5)
         # can initialise with a chain
@@ -510,17 +539,17 @@ class CommonMinimizerTest(unittest.TestCase):
 
     def test_emcee_reuse_sampler(self):
         if not HAS_EMCEE:
-            return True
+            return
 
         self.mini.emcee(nwalkers=20, steps=25)
 
         # if you've run the sampler the Minimizer object should have a _lastpos
         # attribute
-        assert_(hasattr(self.mini, '_lastpos'))
+        assert hasattr(self.mini, '_lastpos')
 
-        # now try and re-use sampler
+        # now try and reuse sampler
         out2 = self.mini.emcee(steps=10, reuse_sampler=True)
-        assert_(out2.chain.shape == (35, 20, 4))
+        assert out2.chain.shape == (35, 20, 4)
 
         # you shouldn't be able to reuse the sampler if nvarys has changed.
         self.mini.params['amp'].vary = False
@@ -557,35 +586,34 @@ class CommonMinimizerTest(unittest.TestCase):
     def test_emcee_output(self):
         # test mcmc output
         if not HAS_EMCEE:
-            return True
+            return
         try:
             from pandas import DataFrame
         except ImportError:
-            return True
+            return
         out = self.mini.emcee(nwalkers=10, steps=20, burn=5, thin=2)
-        assert_(isinstance(out, MinimizerResult))
-        assert_(isinstance(out.flatchain, DataFrame))
+        assert isinstance(out, MinimizerResult)
+        assert isinstance(out.flatchain, DataFrame)
 
         # check that we can access the chains via parameter name
         # print( out.flatchain['amp'].shape[0],  200)
-        assert_(out.flatchain['amp'].shape[0] == 70)
+        assert out.flatchain['amp'].shape[0] == 70
         assert out.errorbars
-        assert_(np.isfinite(out.params['amp'].correl['period']))
+        assert np.isfinite(out.params['amp'].correl['period'])
 
         # the lnprob array should be the same as the chain size
-        assert_(np.size(out.chain)//out.nvarys == np.size(out.lnprob))
+        assert np.size(out.chain)//out.nvarys == np.size(out.lnprob)
 
         # test chain output shapes
         print(out.lnprob.shape, out.chain.shape, out.flatchain.shape)
-        assert_(out.lnprob.shape == (7, 10))
-        assert_(out.chain.shape == (7, 10, 4))
-        assert_(out.flatchain.shape == (70, 4))
+        assert out.lnprob.shape == (7, 10)
+        assert out.chain.shape == (7, 10, 4)
+        assert out.flatchain.shape == (70, 4)
 
-    @dec.slow
     def test_emcee_float(self):
         # test that it works if the residuals returns a float, not a vector
         if not HAS_EMCEE:
-            return True
+            return
 
         def resid(pars, x, data=None):
             return -0.5 * np.sum(self.residual(pars, x, data=data)**2)
@@ -605,11 +633,10 @@ class CommonMinimizerTest(unittest.TestCase):
                               burn=50, thin=10, float_behavior='chi2')
         check_paras(out.params, self.p_true, sig=3)
 
-    @dec.slow
     def test_emcee_seed(self):
         # test emcee seeding can reproduce a sampling run
         if not HAS_EMCEE:
-            return True
+            return
 
         out = self.mini.emcee(params=self.fit_params,
                               nwalkers=100,
@@ -623,11 +650,37 @@ class CommonMinimizerTest(unittest.TestCase):
     def test_emcee_ntemps(self):
         # check for DeprecationWarning when using ntemps > 1
         if not HAS_EMCEE:
-            return True
+            return
 
         with pytest.raises(DeprecationWarning):
             _ = self.mini.emcee(params=self.fit_params, ntemps=5)
 
+    def test_emcee_custom_pool(self):
+        # tests use of a custom pool
+
+        if not HAS_EMCEE:
+            return
+
+        global emcee_counter
+        emcee_counter = 0
+
+        class my_pool:
+            def map(self, f, arg):
+                global emcee_counter
+                emcee_counter += 1
+                return map(f, arg)
+
+        def cost_fun(params, **kwargs):
+            return rosen_der([params['a'], params['b']])
+
+        params = Parameters()
+        params.add('a', 1, min=-5, max=5, vary=True)
+        params.add('b', 1, min=-5, max=5, vary=True)
+
+        fitter = Minimizer(cost_fun, params)
+        fitter.emcee(workers=my_pool(), steps=1000, nwalkers=100)
+        assert emcee_counter > 500
+
 
 def residual_for_multiprocessing(pars, x, data=None):
     # a residual function defined in the top level is needed for
@@ -642,4 +695,4 @@ def residual_for_multiprocessing(pars, x, data=None):
     model = amp*np.sin(shift + x/per) * np.exp(-x*x*decay*decay)
     if data is None:
         return model
-    return (model - data)
+    return model - data
